-- Whack-a-mole Mission Script
-- based on splash damage script


-- parameter Settings
local script_enable = 1
local refreshRate = 0.1
-- Global Local variables
local e = {}
local tracked_weapons = {}
-- ============================= Delay Function
function delay()
  return nil
end

-- ================================ Calculation Functions ==============================
local function getDistance(point1, point2)
  local x1 = point1.x
  local y1 = point1.y
  local z1 = point1.z
  local x2 = point2.x
  local y2 = point2.y
  local z2 = point2.z
  local dX = math.abs(x1-x2)
  local dZ = math.abs(z1-z2)
  local distance = math.sqrt(dX*dX + dZ*dZ)
  return distance
end

local function getDistance3D(point1, point2)
  local x1 = point1.x
  local y1 = point1.y
  local z1 = point1.z
  local x2 = point2.x
  local y2 = point2.y
  local z2 = point2.z
  local dX = math.abs(x1-x2)
  local dY = math.abs(y1-y2)
  local dZ = math.abs(z1-z2)
  local distance = math.sqrt(dX*dX + dZ*dZ + dY*dY)
  return distance
end

local function vec3Mag(speedVec)
  local mag = speedVec.x*speedVec.x + speedVec.y*speedVec.y+speedVec.z*speedVec.z
  mag = math.sqrt(mag)
  return mag
end
         
local function lookahead(speedVec)
  local speed = vec3Mag(speedVec)
  local dist = speed * refreshRate * 1.5 
  return dist
end
      
-- ==================== Event Handler  - Adds Weapon to tracked_weapons to initiate tracking =================
  function e:onEvent(event)
    if event.id == 1 then
      local pilot = event.initiator:getName()
      local ordnance = event.weapon
      local tgt = ordnance:getTarget()
      local weapType = ordnance:getTypeName()
      tracked_weapons[event.weapon.id_] = { wpn = ordnance, init = event.initiator:getName(), pos = ordnance:getPoint(), 
           dir = ordnance:getPosition().x, name = ordnance:getTypeName(), 
           speed = ordnance:getVelocity(), cat = ordnance:getCategory() 
         }
      if pilot == "Sidekick" or pilot =="Sidekick-1" then -- Fired By Player
        if weapType == "HYDRA_70_M156" then -- and it's a WP rocket
          trigger.action.outText("Mark Away",10)   
        end
      else -- Shot By Strike A/C
        trigger.action.outText("Bombs Away",10)
        trigger.action.outSound('204521__redoper__roger-beep.ogg')
      end
    end
  end
-- ================ Assign Actions to Strike Group ============
function assignTarget(tgtLocn, strikeGrp)
  local target = {}
  target.point = {x = tgtLocn.x, y = tgtLocn.z}
  target.weaponType = 16
  target.expend = "Two"
  target.attackQty = 1
  target.attackQtyLimit = true
  target.altitude = 100
  target.altitudeEnabled = true
  local engage = {id = 'Bombing', params = target}
  Group.getByName(strikeGrp):getController():setTask(engage) -- give the strike a/c the bombing task
end
  
  
-- ================ Main Weapons Tracker and Result Handler ===========
  function track_wpns()
    for wpn_id_, wpnData in pairs(tracked_weapons) do 
      -- trigger.action.outText("Weapon Dropped by: " .. wpnData.init, 20)
      -- trigger.action.outText("Weapon Type: " .. wpnData.name, 20)
      if wpnData.wpn:isExist() then  -- just update speed, position and direction.
        wpnData.pos = wpnData.wpn:getPosition().p
        wpnData.dir = wpnData.wpn:getPosition().x
        wpnData.speed = wpnData.wpn:getVelocity()
      else -- wpn no longer exists, must be dead.
        -- trigger.action.outText("Weapon impacted", 1)
        local ip = land.getIP(wpnData.pos, wpnData.dir, lookahead(wpnData.speed))  -- terrain intersection point with weapon's nose.  
        local impactPoint
        if not ip then -- use last calculated IP
          impactPoint = wpnData.pos
          --trigger.action.outText("Impact Point:\nPos X: " .. impactPoint.x .. "\nPos Z: " .. impactPoint.z, 20)
        else -- use intersection point
          impactPoint = ip
          --trigger.action.outText("Impact Point:\nPos X: " .. impactPoint.x .. "\nPos Z: " .. impactPoint.z, 20)
        end
        tracked_weapons[wpn_id_] = nil -- remove from tracked weapons first. 
        if wpnData.name == "HYDRA_70_M156" then -- If it was a mark round
          trigger.action.effectSmokeBig(impactPoint,1,1) -- add a smoke effect - which burns beyond WP duration
          assignTarget(impactPoint, strikeElem) -- Assign it as a bombing target to the next ac
          if strikeElem == 'Aerial-1' then
            trigger.action.outText('Switchblade Lead has the mark!',20) -- Lead was given the target
            trigger.action.outSound('204521__redoper__roger-beep.ogg')
            strikeElem = 'Aerial-3'                                     -- Make Two the next one in line
          else
            trigger.action.outText('Two has the mark!',20)              -- Two was given the target
            trigger.action.outSound('204521__redoper__roger-beep.ogg')
            strikeElem = 'Aerial-1'                                     -- Make Lead the next in line
          end
        end
      end
    end
  end


-- =============== Protected Call and Timer Schedule to Run The Whole Thing ===================     
local function protectedCall(...)
  local status, retval = pcall(...)
  if not status then
    env.warning("Script error... gracefully caught! " .. retval, true)
  end
end      
      
if (script_enable == 1) then
  strikeElem = "Aerial-1"
  trigger.action.outText("Whack-a-Mole Mission Script Running",20)
  timer.scheduleFunction(function() 
      protectedCall(track_wpns)
      return timer.getTime() + refreshRate
    end, 
    {}, 
    timer.getTime() + refreshRate
  )
  world.addEventHandler(e)
end
   
    
    
